class UnresolvedAlgorithmTypeError(Exception):
    pass


class UnusedStrictUrlParameterError(Exception):
    pass


class IncorrectUrlMethodError(Exception):
    pass


class RobokassaInterfaceError(Exception):
    pass
